import os
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
from configuration.config import DATASETH_PATH

def battles_per_year(king_name, dataset=pd.read_csv(DATASETH_PATH)):
    years = dataset['year'].unique()
    years.sort()
    battles_per_year_dict = {year: 0 for year in years}

    king_columns = ["attacker_king", "defender_king"]

    for year in years:
        yearly_data = dataset[dataset['year'] == year]
        for col in king_columns:
            battles_per_year_dict[year] += yearly_data[col].apply(lambda x: x == king_name).sum()

    years = list(battles_per_year_dict.keys())
    battles = list(battles_per_year_dict.values())

    plt.figure(figsize=(10, 5))
    years_str = list(map(str, years))
    plt.bar(years_str, battles, color='r', width=0.2)
    plt.title(f'Количество битв с участием {king_name} по годам')
    plt.xlabel('Год')
    plt.ylabel('Количество битв')
    plt.grid(True)

    plt.gca().set_facecolor('white')
    dir_name = f"{king_name}"
    file_name = "battles_per_year.png"
    path = os.path.join(dir_name, file_name)
    os.makedirs(dir_name, exist_ok=True)
    plt.savefig(path)

def king_attack_defense(king_name, dataset=pd.read_csv(DATASETH_PATH)):
    attacks = dataset['attacker_king'].apply(lambda x: x == king_name).sum()
    defenses = dataset['defender_king'].apply(lambda x: x == king_name).sum()

    roles = ['Атака', 'Защита']
    counts = [attacks, defenses]

    plt.figure(figsize=(8, 5))
    plt.bar(roles, counts, color=['blue', 'green'])
    plt.title(f'Роль в битвах для {king_name}')
    plt.xlabel('Роль')
    plt.ylabel('Число битв')
    plt.ylim(0, max(counts) + 5)
    plt.grid(axis='y', linestyle='--', alpha=0.7)

    dir_name = f"{king_name}"
    file_name = "king_attack_defense.png"
    path = os.path.join(dir_name, file_name)
    os.makedirs(dir_name, exist_ok=True)
    plt.savefig(path)

def army_size(king_name, dataset=pd.read_csv(DATASETH_PATH)):
    king_battles = dataset[(dataset['attacker_king'] == king_name) | (dataset['defender_king'] == king_name)].copy()

    king_battles['army_size'] = np.where(king_battles['attacker_king'] == king_name,
                                         king_battles['attacker_size'],
                                         king_battles['defender_size'])

    king_battles['opponent_king'] = np.where(king_battles['attacker_king'] == king_name,
                                             king_battles['defender_king'],
                                             king_battles['attacker_king'])
    king_battles = king_battles.dropna(subset=['army_size', 'opponent_king'])

    king_battles.reset_index(inplace=True)
    king_battles['battle_number'] = king_battles.index + 1

    plt.figure(figsize=(12, 6))

    opponent_kings = king_battles['opponent_king'].unique()
    colors = plt.cm.get_cmap('tab10', len(opponent_kings))

    for i, opponent_king in enumerate(opponent_kings):
        opponent_battles = king_battles[king_battles['opponent_king'] == opponent_king]
        plt.plot(opponent_battles['battle_number'], opponent_battles['army_size'], marker='o', linestyle='-',
                 color=colors(i), label=opponent_king)

        for j in range(len(opponent_battles)):
            plt.text(opponent_battles['battle_number'].iloc[j], opponent_battles['army_size'].iloc[j],
                     f"{opponent_battles['year'].iloc[j]}", fontsize=9, ha='left')

    plt.title(f'Размер армии короля {king_name} в различных битвах')
    plt.xlabel('Номер битвы')
    plt.ylabel('Размер армии')
    plt.legend(title='Opponent Kings')
    plt.grid(True)
    plt.gca().set_facecolor('white')
    dir_name = f"{king_name}"
    file_name = "army_size.png"
    path = os.path.join(dir_name, file_name)
    os.makedirs(dir_name, exist_ok=True)
    plt.savefig(path)

def main():
    army_size("Balon Greyjoy")

if __name__ == "__main__":
    main()
